#define UNICODE

#include <windows.h>
#include <windowsx.h>
#include "resource.h"

// -------------------------------------------
//  
// -------------------------------------------
LRESULT WINAPI
WndProc(HWND hWnd, UINT msg, WPARAM wParam,
  LPARAM lParam);

void WndProc_OnPaint(HWND hWnd);
BOOL WndProc_OnCreate(HWND hWnd, 
  LPCREATESTRUCT lpCreateStruct);

void WndProc_OnDestroy(HWND hWnd);

BOOL GetFont(HWND hWnd, LOGFONT *lf, 
  CHOOSEFONT *cf);

void WndProc_OnCommand(HWND hWnd, int id,
  HWND hwndctl, UINT codeNotufy);

void MenuItemsUncheck(HWND hWnd);

// -------------------------------------------
//  
// -------------------------------------------
HINSTANCE hInst;
TCHAR szAppName[] = TEXT("Window");
TCHAR szAppTitle[] = TEXT(" ");

static CHOOSEFONT cf;
static LOGFONT lf;
static HFONT hfont, hfOldFont;
UINT uCurrentCharset = DEFAULT_CHARSET;

TCHAR szChars[] = TEXT(": AaBbCc ");
TCHAR szBuf[256];
TCHAR szFontInfo[256];

// -------------------------------------------
// WinMain
// -------------------------------------------
int APIENTRY 
WinMain(HINSTANCE hInstance, 
  HINSTANCE hPrevInstance, LPSTR lpCmdLine,
  int nCmdShow)
{
  WNDCLASSEX wc;
  HWND hWnd;
  MSG msg;
  
  hInst = hInstance;

  hWnd = FindWindow(szAppName, NULL);
  if(hWnd)
  {
    if(IsIconic(hWnd))
      ShowWindow(hWnd, SW_RESTORE);

    SetForegroundWindow(hWnd);
    return FALSE;
  }

  memset(&wc, 0, sizeof(wc));
  
  wc.cbSize = sizeof(WNDCLASSEX);
  wc.hIconSm = LoadImage(hInst,
    MAKEINTRESOURCE(IDI_APPICON_SM),
    IMAGE_ICON, 16, 16, 0);
  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc = (WNDPROC)WndProc;
  wc.cbClsExtra  = 0;
  wc.cbWndExtra  = 0;
  wc.hInstance = hInst;
  wc.hIcon = LoadImage(hInst,
    MAKEINTRESOURCE(IDI_APPICON), 
    IMAGE_ICON, 32, 32, 0);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = MAKEINTRESOURCE(IDR_MENU);
  wc.lpszClassName = szAppName;

  if(!RegisterClassEx(&wc))
    if(!RegisterClass((LPWNDCLASS)&wc.style))
      return FALSE;
    
  hWnd = CreateWindow(szAppName, szAppTitle,
    WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, 
    CW_USEDEFAULT, 0, NULL, NULL, hInst, NULL);
  if(!hWnd) 
    return(FALSE);

  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);

  CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1),
    ID_FONT_DEFAULTCHARSET,
    MF_BYCOMMAND | MF_CHECKED);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
  return msg.wParam;
}

// -------------------------------------------
// WndProc
// -------------------------------------------
LRESULT WINAPI
WndProc(HWND hWnd, UINT msg, WPARAM wParam,
  LPARAM lParam)
{
  switch(msg)
  {
    HANDLE_MSG(hWnd, WM_CREATE, 
      WndProc_OnCreate);
    HANDLE_MSG(hWnd, WM_DESTROY, 
      WndProc_OnDestroy);
    HANDLE_MSG(hWnd, WM_PAINT, 
      WndProc_OnPaint);
    HANDLE_MSG(hWnd, WM_COMMAND, 
      WndProc_OnCommand);

	default:
    return(DefWindowProc(hWnd, msg, wParam,
      lParam));
  }
}

// -------------------------------------------
// WndProc_OnCommand
// -------------------------------------------
#pragma warning(disable: 4098)
void 
WndProc_OnCommand(HWND hWnd, int id,
  HWND hwndCtl, UINT codeNotify)
{
  switch(id)
  {
    case ID_FILE_EXIT:
    {
      PostQuitMessage(0);
      return 0L;
      break;
    }
    case ID_FONT_SELECT:
    {
      memset(&lf, 0, sizeof(LOGFONT));

      if(GetFont(hWnd, &lf, &cf))
         InvalidateRect(hWnd, NULL, TRUE);
      return 0L;
      break;
    }
    case ID_FONT_DEFAULTCHARSET:
    {
      uCurrentCharset = DEFAULT_CHARSET;
      MenuItemsUncheck(hWnd);
      CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1),
        ID_FONT_DEFAULTCHARSET,
        MF_BYCOMMAND | MF_CHECKED);

      return 0L;
      break;
    }
    case ID_FONT_RUSSIANCHARSET:
    {
      uCurrentCharset = RUSSIAN_CHARSET;
      MenuItemsUncheck(hWnd);
      CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1),
        ID_FONT_RUSSIANCHARSET,
        MF_BYCOMMAND | MF_CHECKED);
      return 0L;
      break;
    }
    case ID_FONT_ANSICHARSET:
    {
      uCurrentCharset = ANSI_CHARSET;
      MenuItemsUncheck(hWnd);
      CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1),
        ID_FONT_ANSICHARSET,
        MF_BYCOMMAND | MF_CHECKED);
      return 0L;
      break;
    }
    case ID_FONT_OEMCHARSET:
    {
      uCurrentCharset = OEM_CHARSET;
      MenuItemsUncheck(hWnd);
      CheckMenuItem(GetSubMenu(GetMenu(hWnd), 1),
        ID_FONT_OEMCHARSET,
        MF_BYCOMMAND | MF_CHECKED);
      return 0L;
      break;
    }
  }
  return FORWARD_WM_COMMAND(hWnd, id, hwndCtl,
    codeNotify, DefWindowProc);
}

// -------------------------------------------
// WndProc_OnPaint
// -------------------------------------------
#pragma warning(disable: 4098)
void WndProc_OnPaint(HWND hWnd)
{
  HDC hdc;
  PAINTSTRUCT ps;
  UINT nCharSet;
  
  hdc = BeginPaint(hWnd, &ps);

  lf.lfOrientation = lf.lfEscapement = 0;
  lf.lfCharSet = uCurrentCharset;

  hfont = CreateFontIndirect(&lf);

  if(hfont)
  {
    nCharSet = GetTextCharset(hdc);

    wsprintf(szFontInfo, TEXT("Charset: %u "), nCharSet);

    switch(nCharSet)
    {
      case ANSI_CHARSET:
      {
        lstrcat(szFontInfo, TEXT("(ANSI_CHARSET)"));
        break;
      }
      case OEM_CHARSET:
      {
        lstrcat(szFontInfo, TEXT("(OEM_CHARSET)"));
        break;
      }
      case RUSSIAN_CHARSET:
      {
        lstrcat(szFontInfo, TEXT("(RUSSIAN_CHARSET)"));
        break;
      }
      case EASTEUROPE_CHARSET:
      {
        lstrcat(szFontInfo, TEXT("(EASTEUROPE_CHARSET)"));
        break;
      }
      case DEFAULT_CHARSET:
      {
        lstrcat(szFontInfo, TEXT("(DEFAULT_CHARSET)"));
        break;
      }
    }

    TextOut(hdc, 20, 20, szFontInfo, lstrlen(szFontInfo));

    hfOldFont = SelectFont(hdc, hfont);
    GetTextFace(hdc, 80, szBuf);
    lstrcat(szBuf, szChars);
    SetTextColor(hdc, cf.rgbColors);

    TextOut(hdc, 20, 40, szBuf, lstrlen(szBuf));

    SelectFont(hdc, hfOldFont);
    DeleteFont(hfont);
  }
  EndPaint(hWnd, &ps);
  return 0;
}

// -------------------------------------------
// WndProc_OnCreate
// -------------------------------------------
BOOL WndProc_OnCreate(HWND hWnd, 
   LPCREATESTRUCT lpCreateStruct)
{
  return TRUE;
}

// -------------------------------------------
// WndProc_OnDestroy
// -------------------------------------------
#pragma warning(disable: 4098)
void WndProc_OnDestroy(HWND hWnd)
{
  PostQuitMessage(0);
  return FORWARD_WM_DESTROY(hWnd, DefWindowProc);
}

// -------------------------------------------
// GetFont
// -------------------------------------------
BOOL GetFont(HWND hWnd, LOGFONT *lf,
  CHOOSEFONT *cf)
{
  LPSTR szFontStyle[LF_FACESIZE];

  memset(cf, 0, sizeof(CHOOSEFONT));
  cf->lStructSize = sizeof(CHOOSEFONT);
  cf->hwndOwner = hWnd;
  cf->lpLogFont = lf;
  cf->Flags = CF_SCREENFONTS | CF_USESTYLE |
    CF_EFFECTS;
  cf->lCustData = 0L;
  cf->rgbColors = RGB(0,0,0);
  cf->lpfnHook = NULL;
  cf->lpTemplateName = (LPCTSTR)NULL;
  cf->hInstance = hInst;
  cf->lpszStyle = (LPTSTR)szFontStyle;
  cf->nFontType = SCREEN_FONTTYPE;
  cf->nSizeMin = 0;
  cf->nSizeMax = 0;
  return ChooseFont(cf);
} 

// -------------------------------------------
// MenuItemsUncheck
// -------------------------------------------
void MenuItemsUncheck(HWND hWnd)
{
  int i;

  HMENU hmenu = GetSubMenu(GetMenu(hWnd), 1);

  for (i=0; i<GetMenuItemCount(hmenu) ; i++)
  {
    CheckMenuItem(hmenu, i, 
    MF_BYPOSITION | MF_UNCHECKED);
  }
}
